-- Grid Module
-- Made by: I_IBlackI_I (Blackstone#4953 on discord) for FactorioMMO
-- This module devides the world in a grid, with a connecting piece inbetween.

-- Trblz
-- to ensure aligned tracks, make sure that the following settings are even numbers
-- for alignment with rails, make sure (grid-border-bridge)/2 is even too
-- See https://imgur.com/a/wG5XSc3 for planning

global.grid = global.grid or {}
global.grid.seed = 1
global.grid.size = 108 -- 96
global.grid.x_border_width = 10 --9
global.grid.y_border_width = 10 --9	
global.grid.x_bridge_width = 6 -- 4 no need for this: width * 1.5 ??
global.grid.y_bridge_width = 6 -- 4



local function normalize(n) --keep numbers at (positive) 32 bits
	return n % 0x80000000
end

-- Grid Ore Module
-- Made by: I_IBlackI_I (Blackstone#4953 on discord) for FactorioMMO
-- This module is an extention to the grid module and is able to place ores / oil in certain "Grid chunks"
global.grid_ore = global.grid_ore or {}
global.grid_ore.resource_chance = 80 --80
global.grid_ore.ore_start_amount = 225
global.grid_ore.ore_random_addition_amount = 450
global.grid_ore.oil_start_amount = 100000
global.grid_ore.oil_random_addition_amount = 200000
global.grid_ore.oil_spout_chance = 1



function grid_ore_place_ore_in_grid_chunck(location, ore)
	
	local xoffset = (math.floor(location.x/global.grid.size))*global.grid.size
	local yoffset = (math.floor(location.y/global.grid.size))*global.grid.size
	local ore_amount = 0
	local x_border_width = global.grid.x_border_width
	local y_border_width = global.grid.y_border_width
	local grid_size = global.grid.size 
	local distance_factor = math.log(math.max(math.abs(location.x/grid_size), math.abs(location.y/grid_size), 1.1))^2
	local random_factor = distance_factor / 10
	
	for j=y_border_width,grid_size-1 do
		for i=x_border_width,grid_size-1 do
		-- prevent ore generated on water or void
		if (not game.surfaces['nauvis'].get_tile(i+xoffset, j+yoffset).collides_with("water-tile")) then
				ore_amount = math.ceil((global.grid_ore.ore_start_amount * distance_factor) + (math.random(global.grid_ore.ore_random_addition_amount)*random_factor))
				game.surfaces['nauvis'].create_entity({name=ore, amount=ore_amount, position={i+xoffset, j+yoffset}})
			end
		end
	end
end

function grid_ore_place_oil_in_grid_chunck(location)
	xoffset = (math.floor(location.x/global.grid.size))*global.grid.size
	yoffset = (math.floor(location.y/global.grid.size))*global.grid.size
	local distance_factor = math.log(math.max(math.abs(location.x/global.grid.size), math.abs(location.y/global.grid.size), 1.1))^2
	local random_factor = distance_factor / 10
	local oil_amount = 0
	for y=global.grid.y_border_width,global.grid.size-1 do
		for x=global.grid.x_border_width,global.grid.size-1 do
			if math.random(200) <= global.grid_ore.oil_spout_chance then
				oil_amount = math.ceil((global.grid_ore.oil_start_amount * distance_factor) + (math.random(global.grid_ore.oil_random_addition_amount)*random_factor))
				game.surfaces["nauvis"].create_entity({name="crude-oil", amount=oil_amount, position={x+xoffset, y+yoffset}})
			end
		end
	end
end

function grid_ore_generate_resources(location)
	if(math.random(global.grid_ore.resource_chance ) == 1) then
		rndm = math.random(13)-1
		if(rndm <= 1) then
			grid_ore_place_ore_in_grid_chunck(location, "stone")
		elseif (rndm >= 2 and rndm <= 4) then
			grid_ore_place_ore_in_grid_chunck(location, "iron-ore")
		elseif (rndm >= 5 and rndm <= 7) then
			grid_ore_place_ore_in_grid_chunck(location, "copper-ore")
		elseif (rndm >= 8 and rndm <= 10) then
			grid_ore_place_ore_in_grid_chunck(location, "coal")
		elseif (rndm == 11) then
			grid_ore_place_oil_in_grid_chunck(location)
		elseif (rndm == 12) then
			grid_ore_place_ore_in_grid_chunck(location, "uranium-ore")
		end
	end
end

function grid_place_ore_in_chunk(area)
	local topleftx = area.left_top.x
	local toplefty = area.left_top.y
	local bottomrightx = area.right_bottom.x
	local bottomrighty = area.right_bottom.y
	local location ={x=topleftx,y=topleftx}
	if(math.random(global.grid_ore.resource_chance ) == 1) then
		rndm = math.random(13)-1
		if(rndm <= 1) then
			grid_ore_place_ore_in_grid_chunck(location, "stone")
		elseif (rndm >= 2 and rndm <= 4) then
			grid_ore_place_ore_in_grid_chunck(location, "iron-ore")
		elseif (rndm >= 5 and rndm <= 7) then
			grid_ore_place_ore_in_grid_chunck(location, "copper-ore")
		elseif (rndm >= 8 and rndm <= 10) then
			grid_ore_place_ore_in_grid_chunck(location, "coal")
		elseif (rndm == 11) then
			grid_ore_place_oil_in_grid_chunck(location)
		elseif (rndm == 12) then
			grid_ore_place_ore_in_grid_chunck(location, "uranium-ore")
		end
	end
end


function grid_replace_tiles_in_chunk(area)

	local topleftx = area.left_top.x
	local toplefty = area.left_top.y
	local bottomrightx = area.right_bottom.x
	local bottomrighty = area.right_bottom.y
	local tileTable = {}
	
	local x_border_width = global.grid.x_border_width
	local y_border_width = global.grid.y_border_width
	local x_bridge_width = global.grid.x_bridge_width
	local y_bridge_width = global.grid.y_bridge_width
	local grid_size = global.grid.size 
	
	-- randomly remove a bridge between grids.
	if math.random(10)<2 then
		x_bridge_width = 0
	end
	if math.random(10)<2 then
		y_bridge_width = 0
	end
	
	
	for y = toplefty,bottomrighty - 1 do
        for x = topleftx,bottomrightx - 1 do
			i = x%grid_size -- x in grid
            j = y%grid_size -- y in grid
            -- void column
			if (j < y_border_width) and (i < (grid_size + x_border_width - x_bridge_width) / 2 or i >= (grid_size + x_border_width + x_bridge_width) / 2) then
						table.insert(tileTable,{ name = "out-of-map", position = {x, y}})
			end
            -- void row
			if (i < x_border_width and j > y_border_width - 1) and (j < (grid_size + y_border_width - y_bridge_width) / 2 or j >= (grid_size + y_border_width + y_bridge_width) / 2) then
					table.insert(tileTable,{ name = "out-of-map", position = {x, y}})
			end
        end
    end
	
	game.surfaces['nauvis'].set_tiles(tileTable)
	--Suppress normal resource generation.  Doesn't work because the grid ore is spawned in one big pass.
	if 1 == 2 then
		for _, ore in pairs(game.surfaces[1].find_entities_filtered{type="resource", area=area}) do
			ore.destroy()
		end
	end
	grid_ore_generate_resources({x = topleftx, y=toplefty})
end


Event.register(defines.events.on_chunk_generated, function(event)
--	grid_place_ore_in_chunk(event.area)
	grid_replace_tiles_in_chunk(event.area)
	
end)

Event.register(defines.events.on_player_created, function(event)
    local p = game.players[event.player_index]
    local surface = game.surfaces["nauvis"]
	p.teleport(surface.find_non_colliding_position("player", {x = math.floor(global.grid.size/2), y = math.floor(global.grid.size/2)}, 3, 1))
end)

Event.register('on_init', function(event)
	--global.grid_module.seed = normalize(os.time())
	--randomseed(361625)
	
end)
